import sys
import os
from config import config

# 导入必要的模块
try:
    import numpy as np
    import essentia
    import essentia.standard as es
    from xml.etree.ElementTree import Element, SubElement, Comment, tostring
    from xml.dom import minidom
    from math import gcd
    import hashlib
    import datetime
    print("所有模块导入成功")
except Exception as e:
    print(f"模块导入错误: {str(e)}")
    sys.exit(1)

class BeatDetector:
    def __init__(self, audio_path=None):
        self.audio_path = audio_path
        self.audio = None
        
        # 节拍控制参数
        self.simplification_factor = 1  # 节拍简化倍率
        self.beat_offset_count = 0  # 节拍偏移数量（整数，正数向后移动，负数向前移动）
        self.current_bpm = 120.0  # 当前BPM
        
        # 预设参数
        self.presets = {
            'default': {  # 通用预设
                'fixed': {
                    'window_size': 50,
                    'energy_threshold': 2.0,
                    'min_interval': 0.1
                },
                'drum': {
                    'window_size': 30,
                    'energy_threshold': 1.1,
                    'min_interval': 0.05
                }
            },
            'electronic': {  # 电子音乐
                'fixed': {
                    'window_size': 40,
                    'energy_threshold': 1.8,
                    'min_interval': 0.08
                },
                'drum': {
                    'window_size': 25,
                    'energy_threshold': 1.0,
                    'min_interval': 0.03
                }
            },
            'rock': {  # 摇滚
                'fixed': {
                    'window_size': 60,
                    'energy_threshold': 2.2,
                    'min_interval': 0.12
                },
                'drum': {
                    'window_size': 35,
                    'energy_threshold': 1.3,
                    'min_interval': 0.06
                }
            },
            'pop': {  # 流行音乐
                'fixed': {
                    'window_size': 45,
                    'energy_threshold': 1.9,
                    'min_interval': 0.09
                },
                'drum': {
                    'window_size': 28,
                    'energy_threshold': 1.2,
                    'min_interval': 0.04
                }
            },
            'jazz': {  # 爵士乐
                'fixed': {
                    'window_size': 70,
                    'energy_threshold': 1.7,
                    'min_interval': 0.13
                },
                'drum': {
                    'window_size': 38,
                    'energy_threshold': 1.4,
                    'min_interval': 0.07
                }
            },
            'classical': {  # 古典音乐
                'fixed': {
                    'window_size': 80,
                    'energy_threshold': 1.5,
                    'min_interval': 0.15
                },
                'drum': {
                    'window_size': 40,
                    'energy_threshold': 2.0,
                    'min_interval': 0.1
                }
            },
            'hiphop': {  # 嘻哈
                'fixed': {
                    'window_size': 35,
                    'energy_threshold': 2.1,
                    'min_interval': 0.07
                },
                'drum': {
                    'window_size': 22,
                    'energy_threshold': 0.9,
                    'min_interval': 0.025
                }
            }
        }
        
        # 使用默认预设
        self.current_preset = 'default'
        preset = self.presets[self.current_preset]
        self.fixed_params = preset['fixed'].copy()
        self.drum_params = preset['drum'].copy()
        
        if audio_path:
            self.audio = es.MonoLoader(filename=audio_path)()
        
    def set_audio(self, audio_data):
        """直接设置音频数据"""
        self.audio = audio_data
        
    def set_parameters(self, fixed_params=None, drum_params=None):
        """设置检测参数"""
        if fixed_params:
            self.fixed_params.update(fixed_params)
        if drum_params:
            self.drum_params.update(drum_params)
        
    def set_preset(self, preset_name):
        """设置预设参数"""
        if preset_name in self.presets:
            self.current_preset = preset_name
            preset = self.presets[preset_name]
            self.fixed_params = preset['fixed'].copy()
            self.drum_params = preset['drum'].copy()
            return True
        return False
    
    def detect_beats(self):
        # 1. 使用多特征节拍追踪器
        beat_tracker = es.BeatTrackerMultiFeature()
        beats, confidence = beat_tracker(self.audio)
        
        # 计算 BPM
        if len(beats) > 1:
            intervals = np.diff(beats)
            mean_interval = np.mean(intervals)
            self.current_bpm = 60.0 / mean_interval
        else:
            self.current_bpm = 120.0
            
        # 检测拍号
        time_signature = self.detect_time_signature(beats)
        
        # 2. 使用 Percival2014 进行鼓点检测
        # 预处理
        # 根据窗口大小计算帧大小和跳跃大小
        frame_size = int(self.drum_params['window_size'] * 44.1)  # 毫秒转换为采样点
        # 确保帧大小是2的幂
        frame_size = 2 ** int(np.ceil(np.log2(frame_size)))
        hop_size = frame_size // 4
        w = es.Windowing(type='hann')
        spec = es.Spectrum()
        mels = es.MelBands(numberBands=40)  # 使用梅尔频带
        
        # 存储特征
        mel_bands = []
        
        # 计算梅尔频带能量
        for frame in es.FrameGenerator(self.audio, frameSize=frame_size, hopSize=hop_size):
            windowed = w(frame)
            spectrum = spec(windowed)
            bands = mels(spectrum)
            mel_bands.append(bands)
        
        mel_bands = np.array(mel_bands)
        
        # 计算差分
        diff = np.diff(mel_bands, axis=0)
        diff = np.concatenate([np.zeros((1, diff.shape[1])), diff])
        
        # 只保留正向变化（上升沿）
        diff[diff < 0] = 0
        
        # 计算每帧的打击乐得分
        perc_score = np.sum(diff, axis=1)
        
        # 寻找局部最大值
        peaks = []
        window_size = int(self.drum_params['window_size'] * 44100 / (1000 * hop_size))
        
        for i in range(window_size, len(perc_score) - window_size):
