import sys
import os
import traceback
import datetime
import subprocess

from config import config

# 导入必要的模块
try:
    from PyQt6.QtWidgets import (
        QApplication, QMainWindow, QWidget, QVBoxLayout,
        QHBoxLayout, QPushButton, QLabel, QFileDialog,
        QSlider, QSpacerItem, QSizePolicy, QMessageBox,
        QProgressBar, QGroupBox, QFormLayout, QSpinBox,
        QDoubleSpinBox, QTextEdit, QComboBox, QCheckBox,
        QGridLayout
    )
    from PyQt6.QtCore import (
        Qt, QTimer, pyqtSignal, QLibraryInfo, QMimeData,
        QObject, QUrl, QSize
    )
    from PyQt6.QtGui import (
        QIcon, QDragEnterEvent, QDropEvent, QMouseEvent,
        QPen, QColor, QPainterPath, QPainter, QKeySequence,
        QShortcut
    )
    from PyQt6.QtMultimedia import QMediaPlayer, QAudioOutput, QMediaDevices
    from PyQt6.QtMultimediaWidgets import QVideoWidget
    import pyqtgraph as pg
    import numpy as np
    import essentia
    import essentia.standard
    from beat_detector_essentia import BeatDetector

    # 设置 essentia 日志级别
    essentia.log.infoActive = False
    essentia.log.warningActive = False

    print("所有模块导入成功")
except Exception as e:
    print(f"模块导入错误: {str(e)}")
    traceback.print_exc()
    sys.exit(1)

# 在导任模块之前设置路径
# setup_library_path()

try:
    print("开始导入模块...")
    
    # 1. 首先导入 PyQt6 相关模块
    print("导入 PyQt6...")
    from PyQt6.QtCore import Qt, QSize, QTimer, QThread, pyqtSignal, QMimeData, QObject, QUrl
    from PyQt6.QtWidgets import (
        QApplication, QMainWindow, QLabel, QPushButton, 
        QVBoxLayout, QHBoxLayout, QWidget, QFileDialog,
        QSlider, QMessageBox, QGroupBox, QFormLayout,
        QSpinBox, QDoubleSpinBox, QTextEdit, QComboBox, 
        QProgressBar, QGraphicsScene, QGraphicsView, QCheckBox, QGridLayout
    )
    from PyQt6.QtGui import QDragEnterEvent, QDropEvent, QMouseEvent, QPen, QColor, QPainterPath, QPainter, QKeySequence, QShortcut
    from PyQt6.QtMultimedia import QMediaPlayer, QAudioOutput, QAudioFormat
    
    # 3. 最后导入 numpy
    print("导入 numpy...")
    import numpy as np
    
    # 4. 最后导入 pyqtgraph
    print("导入 pyqtgraph...")
    import pyqtgraph as pg
    import essentia
    import essentia.standard
    from beat_detector_essentia import BeatDetector

    # 设置 essentia 志级别为 ERROR，只示错误信息
    essentia.log.infoActive = False
    essentia.log.warningActive = False

except ImportError as e:
    print(f"导入模块时出错: {e}")
    print("详细错误信息:")
    traceback.print_exc()
    sys.exit(1)

def setup_pyqtgraph_resources():
    """设置 pyqtgraph 资源路径"""
    try:
        # 如果是打包后的环境
        if hasattr(sys, '_MEIPASS'):
            pg_path = os.path.join(sys._MEIPASS, 'pyqtgraph')
            if os.path.exists(pg_path):
                # 加 pyqtgraph 资源
                pg.setConfigOptions(imageAxisOrder='row-major')
                # 设置其他必要的配置...
    except Exception as e:
        print(f"设置 pyqtgraph 资源路径时出错: {e}")
def get_resource_path(relative_path):
    """获取资源文件的绝对路径"""
    return config.get_resource_path(relative_path)

class DropArea(QLabel):
    """可拖放音频文件的区域"""
    
    file_dropped = pyqtSignal(str)
    
    def __init__(self):
        super().__init__()
        self.setMinimumSize(200, 100)
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.setText("拖放音频文件到这里\n或点击选择文件")
        self.setAcceptDrops(True)
        self.setStyleSheet("""
            QLabel {
                border: 2px dashed #666;
                border-radius: 5px;
                background-color: #2d2d2d;
                color: #ccc;
                padding: 10px;
            }
            QLabel:hover {
                border-color: #999;
                background-color: #333;
            }
        """)
    
    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            event.acceptProposedAction()
    
    def dropEvent(self, event: QDropEvent):
        urls = event.mimeData().urls()
        if urls:
            file_path = urls[0].toLocalFile()
            if file_path.lower().endswith(('.mp3', '.wav', '.m4a', '.aac')):
                self.file_dropped.emit(file_path)
            else:
                QMessageBox.warning(
                    self,
                    "不支持的文件格式",
                    "请选择 MP3, WAV, M4A 或 AAC 格式的音频文件"
                )

    def mousePressEvent(self, event: QMouseEvent):
        if event.button() == Qt.MouseButton.LeftButton:
            file_path, _ = QFileDialog.getOpenFileName(
                self,
                "音频",
                "",
                "音频文件 (*.mp3 *.wav *.m4a *.aac)"
            )
            if file_path:
                self.file_dropped.emit(file_path)

class WaveformView(pg.PlotWidget):
    """音频波形显示组件"""
    
    chunk_ready = pyqtSignal(object, object, int, int)
    position_clicked = pyqtSignal(float)
    
    def __init__(self, parent=None):
        # 确保在初始化之前有 QApplication 实例
        if QApplication.instance() is None:
            raise RuntimeError("QApplication must be created before WaveformView")
        
        super().__init__(parent)
        
        if parent:
            parent.layout().addWidget(self)
        
        self.playing = False
        self.current_position = 0
        self.waveform_data = None
        self.fixed_beats = []
        self.drum_beats = []
        self.dragging = False
        self.last_drag_pos = None
        
        self.setup_plot()
        
        # 添鼠标事件处理
        self.scene().sigMouseClicked.connect(self.on_plot_clicked)
        
        # 禁用默认的右菜单
        self.setMenuEnabled(False)
        
        # 发送点击位置的信号
        self.position_clicked.connect(self.on_position_clicked)
    
    def setup_plot(self):
        """设置绘图样式和属性"""
        self.setBackground('#2d2d2d')
        self.getPlotItem().showGrid(True, True, alpha=0.3)
        self.getPlotItem().setLabel('left', '度')
        self.getPlotItem().setLabel('bottom', '时 (秒)')
        self.getPlotItem().getAxis('left').setPen('w')
        self.getPlotItem().getAxis('bottom').setPen('w')
        
        # 设置Y轴固定范围
        self.getPlotItem().setYRange(-1.1, 1.1, padding=0)
        self.getPlotItem().getViewBox().setMouseEnabled(y=False)  # 用Y轴缩
        
        # 添加垂直线用显示当前播放位置
        self.position_line = pg.InfiniteLine(
            pos=0, 
            angle=90, 
            pen=pg.mkPen('y', width=1),
            movable=False
        )
        self.addItem(self.position_line)
    
    def wheelEvent(self, event):
        """处理滚轮事件"""
        if self.waveform_data is None:
            return
            
        # 取当前视图围
        view_range = self.getPlotItem().viewRange()
        view_start, view_end = view_range[0]
        view_width = view_end - view_start
        
        # 获取鼠标位置
        mouse_point = self.plotItem.vb.mapSceneToView(event.position())
        mouse_x = mouse_point.x()
        
        # 计算缩放因子
        if event.angleDelta().y() > 0:
            scale_factor = 0.8  # 大
        else:
            scale_factor = 1.2  # 缩小
            
        # 计算新的视图范围
        new_width = view_width * scale_factor
        new_start = mouse_x - (mouse_x - view_start) * scale_factor
        new_end = new_start + new_width
        
        # 限制缩放范围
        if self.waveform_data is not None:
            min_width = 1.0  # 最小显示1秒
            max_width = len(self.waveform_data) / 44100  # 最大显示整音频
            if min_width <= new_width <= max_width:
                self.setXRange(new_start, new_end, padding=0)
    
    def on_plot_clicked(self, event):
        """处理鼠标点击事"""
        if event.button() == Qt.MouseButton.LeftButton:
            # 获取点击位置的时间
            click_pos = self.getPlotItem().vb.mapSceneToView(event.scenePos())
            time_pos = click_pos.x()
            
            # 发送点击位置信号
            self.position_clicked.emit(time_pos)
    
    def on_position_clicked(self, time_pos):
        """处理点击位置信号"""
        # 发送点击位置信号
        self.chunk_ready.emit(
            self.waveform_data,
            time_pos,
            len(self.fixed_beats),
            len(self.drum_beats)
        )
    
    def plot_waveform(self, waveform_data, sample_rate):
        """绘制波形"""
        if waveform_data is None:
            return
            
        self.waveform_data = waveform_data
        self.audio_duration = len(waveform_data) / sample_rate
        
        # 降采样以提高性能
        downsample_factor = max(1, len(waveform_data) // 10000)
        downsampled_data = waveform_data[::downsample_factor]
        time_axis = np.arange(len(downsampled_data)) * downsample_factor / sample_rate
        
        # 清除现有形
        self.clear()
        self.addItem(self.position_line)
        
        # 绘制波形
        self.plot(
            time_axis, 
            downsampled_data, 
            pen=pg.mkPen('w', width=1)
        )
        
        # 置适当的显示范围
        self.setXRange(0, min(10.0, time_axis[-1]))  # 初始显示10秒
        self.setYRange(-1.1, 1.1)
    
    def plot_beats(self, fixed_beats, drum_beats):
        """绘制节拍标记"""
        self.fixed_beats = fixed_beats
        self.drum_beats = drum_beats
        
        # 获取当前Y轴范围
        y_range = self.getViewBox().viewRange()[1]
        y_height = (y_range[1] - y_range[0])
        
        # 清除现有的节拍标记
        for item in self.items():
            if isinstance(item, pg.InfiniteLine) and item != self.position_line:
                self.removeItem(item)
        
        # 绘固定节拍标记（上层
        for beat_time in fixed_beats:
            # 绘线
            line = pg.InfiniteLine(
                pos=beat_time,
                angle=90,
                pen=pg.mkPen('g', width=1.5),
                movable=False
            )
            self.addItem(line)
            
            # 添加小点标记
            scatter = pg.ScatterPlotItem()
            scatter.addPoints(
                [beat_time],
                [y_range[1] * 0.8],
                symbol='o',
                size=8,
                brush=pg.mkBrush('g'),
                pen=None
            )
            self.addItem(scatter)
        
        # 绘制鼓点节拍标记（下层）
        for beat_time in drum_beats:
            # 绘制竖线
            line = pg.InfiniteLine(
                pos=beat_time,
                angle=90,
                pen=pg.mkPen('r', width=1, style=Qt.PenStyle.DashLine),
                movable=False
            )
            self.addItem(line)
            
            # 添加小方块标记
            scatter = pg.ScatterPlotItem()
            scatter.addPoints(
                [beat_time],
                [y_range[0] * 0.8],
                symbol='s',
                size=6,
                brush=pg.mkBrush('r'),
                pen=None
            )
            self.addItem(scatter)
    
    def update_position(self, position):
        """更当前播放位"""
        if position != self.current_position:
            self.current_position = position
            self.position_line.setPos(position)
            
            # 如果在播放状态，自动滚动视图
            if self.playing and not self.dragging:
                view_range = self.getViewBox().viewRange()
                view_width = view_range[0][1] - view_range[0][0]
                
                # 如果播放位置超出视图范围，滚动视图
                if position > view_range[0][1] or position < view_range[0][0]:
                    self.setXRange(
                        position - view_width * 0.1,  # 留出 10% 的空间
                        position + view_width * 0.9,  # 前面显示 90% 的空间
                        padding=0
                    )
    
    def on_mouse_moved(self, pos):
        """处理鼠标移动事件"""
        if not self.dragging:  # 只在非拖动状态下更新位置
            if self.sceneBoundingRect().contains(pos):
                mouse_point = self.getPlotItem().vb.mapSceneToView(pos)
                x = mouse_point.x()
                if hasattr(self, 'audio_duration') and 0 <= x <= self.audio_duration:
                    # 更新时间指示位
                    self.update_position(x)
    
    def mouseDoubleClickEvent(self, event: QMouseEvent):
        """处理鼠标双击事件"""
        if event.button() == Qt.MouseButton.LeftButton:
            # 获取点击位置的时间
            pos = event.pos()
            scene_pos = self.mapToScene(pos)
            view_pos = self.plotItem.vb.mapSceneToView(scene_pos)
            time_pos = view_pos.x()
            
            # 更播放头位置
            self.update_position(time_pos)
            
            # 发送点击位置信号
            self.position_clicked.emit(time_pos)
            
            event.accept()
        else:
            super().mouseDoubleClickEvent(event)

    def mousePressEvent(self, event: QMouseEvent):
        """处鼠标按下事件"""
        if event.button() == Qt.MouseButton.LeftButton:
            # 置拖动状态
            self.dragging = True
            self.last_drag_pos = event.pos()
            event.accept()
        else:
            super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent):
        """处理鼠标移动事件"""
        if self.dragging and self.last_drag_pos is not None:
            # 计算拖动距离
            delta = event.pos().x() - self.last_drag_pos.x()
            
            # 获取当前视图范围
            view_range = self.getViewBox().viewRange()
            view_width = view_range[0][1] - view_range[0][0]
            
            # 计算时间偏移
            time_delta = -delta * view_width / self.width()
            
            # 更新视图范围
            self.setXRange(
                view_range[0][0] + time_delta,
                view_range[0][1] + time_delta,
                padding=0
            )
            
            # 新拖动位置
            self.last_drag_pos = event.pos()
            event.accept()
        else:
            super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent):
        """处理鼠标释放事件"""
        if event.button() == Qt.MouseButton.LeftButton:
            self.dragging = False
            self.last_drag_pos = None
            event.accept()
        else:
            super().mouseReleaseEvent(event)
            
    def clear_all(self):
        """清除所图形"""
        self.clear()
        self.addItem(self.position_line)
        self.waveform_data = None
        self.fixed_beats = []
        self.drum_beats = []
        self.current_position = 0
        self.position_line.setValue(0)

class ParameterPanel(QGroupBox):
    """节拍检参数制面板"""
    
    parametersChanged = pyqtSignal(dict)
    
    def __init__(self):
        super().__init__("音乐类型")
        self.setup_ui()
        
    def setup_ui(self):
        """设置UI"""
        layout = QVBoxLayout(self)
        
        # 预设选择
        preset_layout = QHBoxLayout()
        preset_layout.addWidget(QLabel("选择音乐类型:"))
        self.preset_combo = QComboBox()
        self.preset_combo.addItems([
            '通用', '电子音乐', '摇滚', '流行音乐',
            '爵士乐', '古典音乐', '嘻哈'
        ])
        self.preset_combo.currentTextChanged.connect(self.on_preset_changed)
        preset_layout.addWidget(self.preset_combo)
        layout.addLayout(preset_layout)
        
        # 添加说明文字
        description = QLabel("选择合适的音乐类型可以获得更好的检测效果")
        description.setWordWrap(True)
        description.setStyleSheet("color: #999; font-size: 12px;")
        layout.addWidget(description)
    
    def on_preset_changed(self, preset_text):
        """处理预设变化"""
        preset_map = {
            '通用': 'default',
            '电子音乐': 'electronic',
            '摇滚': 'rock',
            '流行音乐': 'pop',
            '爵士乐': 'jazz',
            '古典音乐': 'classical',
            '嘻哈': 'hiphop'
        }
        if preset_text in preset_map:
            self.parametersChanged.emit({'preset': preset_map[preset_text]})

class AudioProcessThread(QThread):
    finished = pyqtSignal(dict)
    error = pyqtSignal(str)
    progress = pyqtSignal(float)

    def __init__(self, audio_file):
        super().__init__()
        self.audio_file = audio_file
        self.detector = BeatDetector(audio_file)
    
    def run(self):
        try:
            # 检测节拍
            self.progress.emit(50)
            results = self.detector.detect_beats()
            self.progress.emit(100)
            self.finished.emit(results)
        except Exception as e:
            self.error.emit(str(e))

class AudioPlayer(QObject):
    positionChanged = pyqtSignal(float)
    stateChanged = pyqtSignal(str)
    error = pyqtSignal(str)
    
    def __init__(self):
        super().__init__()
        print("初始化 QMediaPlayer...")
        
        # 创建音频输出
        self.audio_output = QAudioOutput()
        self.audio_output.setVolume(0.5)
        
        # 选择默认音频设备
        audio_devices = QMediaDevices.audioOutputs()
        if audio_devices:
            self.audio_output.setDevice(audio_devices[0])
            print(f"使用音频设备: {audio_devices[0].description()}")
        
        # 创建媒体播放器
        self.player = QMediaPlayer()
        self.player.setAudioOutput(self.audio_output)
        
        # 连接信号
        self.player.positionChanged.connect(self._on_position_changed)
        self.player.playbackStateChanged.connect(self._on_state_changed)
        self.player.errorOccurred.connect(self._on_error)
        self.player.mediaStatusChanged.connect(self._on_media_status_changed)
        
        # 初始化状态
        self.duration = 0.0
        self.current_file = None
    
    def _on_position_changed(self, position):
        """处理位置变化"""
        self.positionChanged.emit(position / 1000.0)  # 转换为秒
    
    def _on_state_changed(self, state):
        """处理播放状态变化"""
        state_map = {
            QMediaPlayer.PlaybackState.StoppedState: "stopped",
            QMediaPlayer.PlaybackState.PlayingState: "playing",
            QMediaPlayer.PlaybackState.PausedState: "paused"
        }
        self.stateChanged.emit(state_map.get(state, "unknown"))
    
    def _on_media_status_changed(self, status):
        """处理媒体状态变化"""
        print(f"媒体状态变化: {status}")
        if status == QMediaPlayer.MediaStatus.LoadedMedia:
            self.duration = self.player.duration() / 1000.0  # 转换为秒
    
    def _on_error(self, error):
        """理错误"""
        error_msg = f"播放器错误: {error} - {self.player.errorString()}"
        print(error_msg)
        self.error.emit(error_msg)

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("节拍检")
        self.resize(800, 600)
        
        # 始化变量
        self.audio_file = None
        self.player = None
        self.fixed_beats = []
        self.drum_beats = []
        self.bpm = 0
        
        # 创建UI组件和设置局
        self.setup_ui()
        self.setup_layout()
        self.setup_style()
    
    def setup_ui(self):
        """创建UI组件"""
        # 创建参数面板
        self.parameter_panel = ParameterPanel()
        self.parameter_panel.parametersChanged.connect(self.on_parameters_changed)
        
        # 创建进度条
        self.progress_bar = QProgressBar()
        self.progress_bar.setVisible(False)
        
        # 创建拖放区域
        self.drop_area = DropArea()
        self.drop_area.file_dropped.connect(self.process_audio_file)
        
        # 创建波形显示
        self.waveform_view = WaveformView()
        self.waveform_view.position_clicked.connect(self.on_position_clicked)
        
        # 创建播放按钮
        self.play_button = QPushButton("播放")
        self.play_button.setEnabled(False)
        self.play_button.clicked.connect(self.play_pause)
        self.play_button.setIcon(QIcon(config.get_icon_path('play.png')))
        self.play_button.setIconSize(QSize(16, 16))
        
        # 创建时间标签
        self.time_label = QLabel("00:00 / 00:00")
        
        # 创建导出按钮
        self.export_button = QPushButton("导出 FCPXML")
        self.export_button.setEnabled(False)
        self.export_button.clicked.connect(self.export_fcpxml)
        self.export_button.setIcon(QIcon(config.get_icon_path('export.png')))
        self.export_button.setIconSize(QSize(16, 16))
        
        # 创建在 FCPX 中打开按钮
        self.open_fcpx_button = QPushButton("在 FCPX 中打开")
        self.open_fcpx_button.setEnabled(False)
        self.open_fcpx_button.clicked.connect(self.open_in_fcpx)
        self.open_fcpx_button.setIcon(QIcon(config.get_icon_path('fcpx.png')))
        self.open_fcpx_button.setIconSize(QSize(16, 16))
        
        # 设置按钮样式
        self.export_button.setStyleSheet("""
            QPushButton {
                background-color: #4CAF50;
                color: white;
                border: none;
                padding: 8px 16px;
                border-radius: 4px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #45a049;
            }
            QPushButton:disabled {
                background-color: #cccccc;
            }
        """)
        
        self.open_fcpx_button.setStyleSheet("""
            QPushButton {
                background-color: #2196F3;
                color: white;
                border: none;
                padding: 8px 16px;
                border-radius: 4px;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #1976D2;
            }
            QPushButton:disabled {
                background-color: #cccccc;
            }
        """)
        
        # 创建节拍点明标签
        self.beat_info_layout = QHBoxLayout()
        fixed_beat_label = QLabel("● 固定节拍")
        fixed_beat_label.setStyleSheet("QLabel { color: #00ff00; }")
        drum_beat_label = QLabel("● 鼓点节拍")
        drum_beat_label.setStyleSheet("QLabel { color: #ff0000; }")
        self.beat_info_layout.addWidget(fixed_beat_label)
        self.beat_info_layout.addWidget(drum_beat_label)
        self.beat_info_layout.addStretch()
        
        # 建导出项
        self.setup_export_settings()
    
    def setup_layout(self):
        """设置布局"""
        central_widget = QWidget()
        main_layout = QVBoxLayout(central_widget)
        
        # 添加参数面
        main_layout.addWidget(self.parameter_panel)
        
        # 添加进度条
        main_layout.addWidget(self.progress_bar)
        
        # 心内区域
        center_widget = QWidget()
        center_layout = QVBoxLayout(center_widget)
        center_layout.setContentsMargins(0, 0, 0, 0)
        center_layout.setSpacing(10)
        
        # 添加拖放区和波形显示到中心区域
        center_layout.addWidget(self.drop_area)
        center_layout.addWidget(self.waveform_view)
        
        # 创建控布局
        control_layout = QHBoxLayout()
        control_layout.setContentsMargins(0, 0, 0, 0)
        control_layout.setSpacing(10)
        control_layout.addWidget(self.play_button)
        control_layout.addWidget(self.time_label)
        control_layout.addStretch()
        
        # 创建导出按钮布局
        export_layout = QHBoxLayout()
        export_layout.addLayout(self.beat_info_layout)
        export_layout.addStretch()
        export_layout.addWidget(self.export_group)
        export_layout.addWidget(self.export_button)
        export_layout.addWidget(self.open_fcpx_button)
        
        # 将所有布添加到中心区域
        center_layout.addLayout(control_layout)
        center_layout.addLayout(export_layout)
        
        # 中心区域添加到主布
        main_layout.addWidget(center_widget)
        
        # 设置中心件
        self.setCentralWidget(central_widget)
    
    def setup_style(self):
        """设置样式"""
        self.setStyleSheet("""
            QMainWindow {
                background-color: #1a1a1a;
                color: white;
            }
            QPushButton {
                background-color: #2d2d2d;
                color: white;
                border: 1px solid #666;
                border-radius: 3px;
                padding: 5px;
                min-width: 80px;
            }
            QPushButton:hover {
                background-color: #3d3d3d;
            }
            QPushButton:disabled {
                background-color: #222;
                color: #666;
            }
            QLabel {
                color: white;
            }
            QTextEdit {
                background-color: #2d2d2d;
                color: white;
                border: 1px solid #666;
            }
            QProgressBar {
                border: 1px solid #666;
                border-radius: 3px;
                text-align: center;
                color: white;
            }
            QProgressBar::chunk {
                background-color: #4a9eff;
            }
        """)
    
    def process_audio_file(self, file_path):
        """处理音频文件"""
        try:
            self.cleanup_current_state()
            
            # 保文件径
            self.audio_file = file_path
            
            # 使用 essentia 加载音频文件
            loader = essentia.standard.MonoLoader(filename=file_path)
            audio_data = loader()
            sample_rate = loader.paramValue('sampleRate')
            
            # 初化测器
            self.current_detector = BeatDetector(file_path)
            
            # 更新波形显示
            self.waveform_view.plot_waveform(audio_data, sample_rate)
            
            print("\n开始初始化播放器...")
            # 初始化播放器
            self.player = QMediaPlayer()
            print(f"播放器状态: {self.player.mediaStatus()}")
            self.audio_output = QAudioOutput()
            print(f"音频输出设备: {self.audio_output.device()}")
            self.audio_output.setVolume(0.5)
            self.player.setAudioOutput(self.audio_output)
            
            # 连接信号
            self.player.positionChanged.connect(self.on_position_changed)
            self.player.playbackStateChanged.connect(self.on_state_changed)
            self.player.errorOccurred.connect(self.on_player_error)
            
            # 加载音频文件
            print(f"加载音频文件: {file_path}")
            self.player.setSource(QUrl.fromLocalFile(file_path))
            print(f"媒体状态: {self.player.mediaStatus()}")
            print(f"播放状态: {self.player.playbackState()}")
            print(f"音频输出状态: {self.audio_output.volume()}")
            print(f"错误状态: {self.player.error()}")
            
            # 更新 UI 状
            self.play_button.setEnabled(True)
            self.export_button.setEnabled(True)
            
            # 设置时间标签
            duration = len(audio_data) / sample_rate
            self.time_label.setText(f"00:00 / {self.format_time(duration)}")
            
            # 开始检测节拍
            self.update_detection()
            
        except Exception as e:
            self.show_error("加载错误", str(e))

    def on_position_changed(self, position):
        """播放位置变化回调"""
        try:
            if not self.waveform_view.dragging:
                # 更新时间指示器
                position_sec = position / 1000.0  # 转换为秒
                self.waveform_view.update_position(position_sec)
                
                # 更新时间标签
                duration = self.player.duration()
                self.time_label.setText(
                    f"{self.format_time(position/1000)} / {self.format_time(duration/1000)}"
                )
        except Exception as e:
            print(f"Position update error: {e}")

    def on_state_changed(self, state):
        """播放状态变化回"""
        if state == QMediaPlayer.PlaybackState.StoppedState:
            self.play_button.setText("播放")
            self.waveform_view.playing = False

    def cleanup_current_state(self):
        """清理当前状态"""
        self.audio_file = None
        self.fixed_beats = []
        self.drum_beats = []
        self.bpm = 0
        
        if self.player:
            self.player.stop()
            self.player = None
        
        self.waveform_view.clear_all()
        self.play_button.setText("播放")
        self.waveform_view.playing = False

    def play_pause(self):
        """播放或暂停音频"""
        if not self.audio_file:
            QMessageBox.warning(self, "警告", "请先加载频文件")
            return

        if self.player.playbackState() == QMediaPlayer.PlaybackState.PlayingState:
            # 停播放
            self.player.pause()
            self.play_button.setText("播放")
            self.waveform_view.playing = False
        else:
            # 继续播放
            self.player.play()
            self.play_button.setText("暂停")
            self.waveform_view.playing = True

    def update_detection(self):
        """更新节拍检测果"""
        if self.current_detector:
            # 测节拍
            result = self.current_detector.detect_beats()
            
            if result:
                self.bpm = result.get('tempo', 0.0)
                self.confidence = result.get('confidence', 0.0)
                self.fixed_beats = result.get('fixed_beats', [])
                self.drum_beats = result.get('drum_beats', [])
                self.beat_type = result.get('beat_type', '4/4')
                
                # 更新波形显示
                self.waveform_view.plot_beats(self.fixed_beats, self.drum_beats)
                
                # 更新状态信息
                self.statusBar().showMessage(
                    f"检测结果更新：BPM: {self.bpm:.1f} ({self.beat_type})  "
                    f"置信度: {self.confidence:.2f}  "
                    f"固定节拍: {len(self.fixed_beats)}  "
                    f"鼓点数: {len(self.drum_beats)}"
                )
            
    def on_volume_changed(self, value):
        """音量变化理"""
        volume = value / 100.0
        if self.player:
            self.player.setVolume(volume)

    def on_player_error(self, error):
        """处理播放器错误"""
        self.show_error("播放器错误", error)

    def show_error(self, title, message):
        """显示错对话框"""
        QMessageBox.critical(self, title, message)

    def setup_export_settings(self):
        """设置导出选项"""
        self.export_group = QGroupBox("导出选项")
        export_layout = QHBoxLayout()
        
        self.export_fixed = QCheckBox("固定节拍")
        self.export_fixed.setChecked(True)
        self.export_drums = QCheckBox("鼓点节拍")
        self.export_drums.setChecked(True)
        
        export_layout.addWidget(self.export_fixed)
        export_layout.addWidget(self.export_drums)
        self.export_group.setLayout(export_layout)

    def open_in_fcpx(self):
        """在 FCPX 中打开导出的文件"""
        if hasattr(self, 'last_exported_file'):
            try:
                subprocess.run(['open', '-a', 'Final Cut Pro', self.last_exported_file])
            except Exception as e:
                self.show_error("开错误", f"无法在 FCPX 中打开件: {str(e)}")
    
    def export_fcpxml(self):
        """导出 FCPXML 文件"""
        if not hasattr(self, 'audio_file') or not self.audio_file:
            return
            
        try:
            # 准备导出数据
            beats_info = {
                'tempo': self.bpm,
                'beat_type': '4/4',
                'fixed_beats': self.fixed_beats if self.export_fixed.isChecked() else [],
                'drum_beats': self.drum_beats if self.export_drums.isChecked() else []
            }
            
            # 生成导出文件
            timestamp = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
            filename = f"{os.path.splitext(os.path.basename(self.audio_file))[0]}_{timestamp}.fcpxml"
            output_dir = os.path.join(config.base_path, 'exports')
            os.makedirs(output_dir, exist_ok=True)
            output_path = os.path.join(output_dir, filename)
            
            # 创建检测器实例并导出
            detector = BeatDetector(self.audio_file)
            detector.export_fcpxml(beats_info, output_path)
            
            # 保存最后导出的文件路径（用于"在 FCPX 中打开"功能）
            self.last_exported_file = output_path
            
            # 启用"在 FCPX 中打开"按钮
            self.open_fcpx_button.setEnabled(True)
            
            # 更新状栏
            self.statusBar().showMessage(f"导出 FCPXML 文件到: {output_path}")
            
        except Exception as e:
            self.show_error("导出错误", str(e))

    def update_progress(self, value):
        """更新进度条"""
        if value < 0:
            value = 0
        elif value > 100:
            value = 100
        self.progress_bar.setValue(int(value))  # 确保值是整

    def format_time(self, seconds):
        """格式化时间为 MM:SS 格式"""
        minutes = int(seconds // 60)
        seconds = int(seconds % 60)
        return f"{minutes:02d}:{seconds:02d}"

    def on_position_clicked(self, time_pos):
        """处理波形视图击件"""
        if self.player:
            # 时间���换为毫秒
            position_ms = int(time_pos * 1000)
            # 设置播放位置
            self.player.setPosition(position_ms)
            # 果没有在播放，开始播放
            if not self.player.playbackState() == QMediaPlayer.PlaybackState.PlayingState:
                self.play_pause()

    def on_parameters_changed(self, params):
        """处理参数变化"""
        if self.current_detector:
            # 更新检测器参数
            self.current_detector.set_parameters(
                fixed_params=params['fixed'],
                drum_params=params['drum']
            )
            
            # 重新检测
            self.update_detection()
            
            # 更新状态栏
            self.statusBar().showMessage("参数已更新，重新检测完成")

def main():
    app = QApplication(sys.argv)
    
    # 设置应程序属性
    app.setApplicationName("BeatDetector")
    app.setOrganizationName("BeatsMark")
    
    # 设置 pyqtgraph 资源路径
    setup_pyqtgraph_resources()
    
    # 创建并显示主窗口
    window = MainWindow()
    window.show()
    
    # 进入事件循环
    sys.exit(app.exec())

if __name__ == '__main__':
    main()
