import sys
import os
from config import config

# 导入必要的模块
try:
    import numpy as np
    import librosa
    import essentia.standard as es
    from xml.etree.ElementTree import Element, SubElement, Comment, tostring
    from xml.dom import minidom
    from math import gcd
    import hashlib
    import datetime
    print("所有模块导入成功")
except Exception as e:
    print(f"模块导入错误: {str(e)}")
    sys.exit(1)

class BeatDetector:
    def __init__(self, audio_path=None):
        self.audio_path = audio_path
        self.audio = None
        
        # 节拍控制参数
        self.simplification_factor = 1  # 节拍简化倍率
        self.beat_offset_count = 0  # 节拍偏移数量（整数，正数向后移动，负数向前移动）
        self.current_bpm = 120.0  # 当前BPM
        
        # 预设参数
        self.presets = {
            'default': {  # 通用预设
                'fixed': {
                    'window_size': 50,
                    'energy_threshold': 2.0,
                    'min_interval': 0.1
                },
                'drum': {
                    'window_size': 30,
                    'energy_threshold': 1.1,
                    'min_interval': 0.05
                }
            },
            'electronic': {  # 电子音乐
                'fixed': {
                    'window_size': 40,
                    'energy_threshold': 1.8,
                    'min_interval': 0.08
                },
                'drum': {
                    'window_size': 25,
                    'energy_threshold': 1.0,
                    'min_interval': 0.03
                }
            },
            'rock': {  # 摇滚
                'fixed': {
                    'window_size': 60,
                    'energy_threshold': 2.2,
                    'min_interval': 0.12
                },
                'drum': {
                    'window_size': 35,
                    'energy_threshold': 1.3,
                    'min_interval': 0.06
                }
            },
            'pop': {  # 流行音乐
                'fixed': {
                    'window_size': 45,
                    'energy_threshold': 1.9,
                    'min_interval': 0.09
                },
                'drum': {
                    'window_size': 28,
                    'energy_threshold': 1.2,
                    'min_interval': 0.04
                }
            },
            'jazz': {  # 爵士乐
                'fixed': {
                    'window_size': 70,
                    'energy_threshold': 1.7,
                    'min_interval': 0.13
                },
                'drum': {
                    'window_size': 38,
                    'energy_threshold': 1.4,
                    'min_interval': 0.07
                }
            },
            'classical': {  # 古典音乐
                'fixed': {
                    'window_size': 80,
                    'energy_threshold': 1.5,
                    'min_interval': 0.15
                },
                'drum': {
                    'window_size': 40,
                    'energy_threshold': 2.0,
                    'min_interval': 0.1
                }
            },
            'hiphop': {  # 嘻哈
                'fixed': {
                    'window_size': 35,
                    'energy_threshold': 2.1,
                    'min_interval': 0.07
                },
                'drum': {
                    'window_size': 22,
                    'energy_threshold': 0.9,
                    'min_interval': 0.025
                }
            }
        }
        
        # 使用默认预设
        self.current_preset = 'default'
        preset = self.presets[self.current_preset]
        self.fixed_params = preset['fixed'].copy()
        self.drum_params = preset['drum'].copy()
        
        if audio_path:
            self.audio = es.MonoLoader(filename=audio_path)()
        
    def set_audio(self, audio_data):
        """直接设置音频数据"""
        self.audio = audio_data
        
    def set_parameters(self, fixed_params=None, drum_params=None):
        """设置检测参数"""
        if fixed_params:
            self.fixed_params.update(fixed_params)
        if drum_params:
            self.drum_params.update(drum_params)
        
    def set_preset(self, preset_name):
        """设置预设参数"""
        if preset_name in self.presets:
            self.current_preset = preset_name
            preset = self.presets[preset_name]
            self.fixed_params = preset['fixed'].copy()
            self.drum_params = preset['drum'].copy()
            return True
        return False
    
    def detect_peaks(self):
        """检测音频中的峰值和节拍"""
        # 使用 essentia 加载音频，这比 librosa 更快
        loader = es.MonoLoader(filename=self.audio_path)
        audio = loader()
        
        # 使用 essentia 的节拍跟踪器
        rhythm_extractor = es.RhythmExtractor2013(method="multifeature")
        bpm, beats, beats_confidence, _, beats_intervals = rhythm_extractor(audio)
        
        # 获取 onset 检测结果
        onset_detector = es.OnsetDetection()
        onset_rate = es.OnsetRate()
        
        # 计算 onset 特征
        w = es.Windowing(type='hann')
        fft = es.FFT()
        c2p = es.CartesianToPolar()
        
        onsets = []
        frame_size = 1024
        hop_size = 512
        for frame in es.FrameGenerator(audio, frameSize=frame_size, hopSize=hop_size):
            windowed_frame = w(frame)
            spectrum = fft(windowed_frame)
            magnitude, phase = c2p(spectrum)
            onset_value = onset_detector(magnitude, phase)
            if onset_value > 0.2:  # 阈值可以根据需要调整
                onsets.append(len(onsets) * hop_size / 44100.0)
        
        # 检测拍号
        time_signature = self.detect_time_signature(beats)
        
        # 应用简化和偏移到固定节拍
        fixed_beats = self.get_displayed_beats(beats)
        
        return {
            'tempo': float(bpm),
            'peaks': onsets,
            'fixed_beats': fixed_beats,
            'drum_beats': onsets,  # 使用 onset 作为鼓点
            'beat_type': time_signature,
            'confidence': float(beats_confidence),
            'simplification_factor': self.simplification_factor,
            'beat_offset_count': self.beat_offset_count
        }
    
    def detect_time_signature(self, beats):
        """检测音乐的拍号"""
        if len(beats) < 4:
            return "4/4"  # 默认值
            
        # 计算相邻拍子之间的间隔
        intervals = np.diff(beats)
        
        # 计算每个小节的长度（假设每个小节的长度相似）
        # 使用自相关来找到重复模式
        def autocorr(x):
            result = np.correlate(x, x, mode='full')
            return result[len(result)//2:]
            
        # 对间隔序列进行自相关
        corr = autocorr(intervals)
        
        # 找到自相关的峰值（除了第一个，它总是最大的）
        peaks = []
        for i in range(1, len(corr)-1):
            if corr[i] > corr[i-1] and corr[i] > corr[i+1]:
                peaks.append(i)
                
        if not peaks:
            return "4/4"
            
        # 取第一个显著峰值作为可能的小节长度
        bar_length = peaks[0] + 1  # +1 因为间隔比拍子少一个
        
        # 分析强拍模式
        if len(beats) >= bar_length * 2:
            # 计算每个位置的平均能量
            energies = []
            window_size = int(0.05 * 44100)  # 50ms窗口
            for beat in beats[:bar_length*2]:
                start = int(beat * 44100)
                end = min(start + window_size, len(self.audio))
                if start < len(self.audio):
                    energy = np.mean(np.abs(self.audio[start:end]))
                    energies.append(energy)
                    
            # 归一化能量
            energies = np.array(energies)
            if len(energies) > 0:
                energies = energies / np.max(energies)
                
                # 分析能量模式
                if bar_length == 3:
                    return "3/4"
                elif bar_length == 6:
                    return "6/8"
                elif bar_length == 2:
                    return "2/4"
                elif bar_length == 5:
                    return "5/4"
                    
        # 如果无法确定，返回默认值
        return "4/4"
    
    def set_simplification_factor(self, factor):
        """设置节拍简化倍率"""
        self.simplification_factor = max(1, int(factor))
        
    def set_beat_offset(self, offset_count):
        """设置节拍偏移数量"""
        self.beat_offset_count = int(offset_count)
        
    def get_displayed_beats(self, beats):
        """获取实际显示的节拍点"""
        if not isinstance(beats, np.ndarray):
            beats = np.array(beats)
            
        if beats.size == 0:
            return beats
            
        # 计算节拍间隔
        beat_interval = 60.0 / self.current_bpm
        
        # 处理向前偏移超出范围的情况
        if self.beat_offset_count < 0:
            # 计算需要添加的前置节拍点数
            extra_beats_count = abs(self.beat_offset_count)
            # 生成前置节拍点
            first_beat = beats[0]
            extra_beats = np.array([first_beat - (i + 1) * beat_interval 
                                  for i in range(extra_beats_count)])
            # 确保不会出现负数时间点
            extra_beats = extra_beats[extra_beats >= 0]
            # 合并节拍点
            beats = np.concatenate([extra_beats[::-1], beats])  # 反转前置节拍点以保持正确顺序
            
        # 应用偏移
        if self.beat_offset_count != 0:
            if self.beat_offset_count > 0:
                # 正向偏移：从指定位置开始
                beats = beats[self.beat_offset_count:]
            else:
                # 负向偏移：已经在上面处理过了
                pass
            
        # 应用简化
        if self.simplification_factor > 1:
            beats = beats[::self.simplification_factor]
            
        return beats
        
    def generate_fixed_beats(self, tempo, beat_type):
        """根据 BPM 生成固定节拍"""
        if tempo <= 0:
            return []
            
        self.current_bpm = tempo
        # 计算拍的时间间隔（秒）
        beat_interval = 60.0 / tempo
        
        # 计算总拍数，包括可能的前置节拍
        total_duration = len(self.audio) / 44100.0
        num_beats = int(total_duration / beat_interval) + abs(min(0, self.beat_offset_count))
        
        # 生成固定间隔的节拍时间点
        beats = np.array([i * beat_interval for i in range(num_beats)])
        
        # 应用简化和偏移
        return self.get_displayed_beats(beats)
    
    def export_fcpxml(self, beats_info, output_path):
        # 设置帧率和时间格式
        frame_duration = "1001/30000s"  # 29.97fps
        
        # 计算精确的音频时长
        total_samples = len(self.audio)
        
        # 计算视频帧时长 (29.97fps)
        total_frames = int(round(total_samples * 30000/(44100 * 1001)))
        sequence_duration = f"{total_frames * 1001}/30000s"
        
        # 创建FCPXML根元素
        root = Element('fcpxml', {'version': '1.11'})
        root.append(Comment('<!DOCTYPE fcpxml>'))
        
        # 添加resources
        resources = SubElement(root, 'resources')
        
        # 添加格式定义
        format_elem = SubElement(resources, 'format', {
            'id': 'r1',
            'name': 'FFVideoFormat3840x2160p2997',
            'frameDuration': frame_duration,
            'width': '3840',
            'height': '2160',
            'colorSpace': '1-1-1 (Rec. 709)'
        })
        
        # 添加音频文件资源
        audio_file = os.path.abspath(self.audio_path)
        uid = hashlib.md5(audio_file.encode()).hexdigest().upper()
        
        asset = SubElement(resources, 'asset', {
            'id': 'r2',
            'name': os.path.basename(self.audio_path),
            'uid': uid,
            'start': '0s',
            'duration': f"{total_samples}/44100s",
            'hasAudio': '1',
            'audioSources': '1',
            'audioChannels': '2',
            'audioRate': '44100'
        })
        
        # 添加 media-rep
        media_rep = SubElement(asset, 'media-rep', {
            'kind': 'original-media',
            'sig': uid,
            'src': f"file://{audio_file}"
        })
        
        # 添加 metadata
        metadata = SubElement(asset, 'metadata')
        md = SubElement(metadata, 'md', {
            'key': 'com.apple.proapps.mio.ingestDate',
            'value': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S +0800')
        })
        
        # 添加library和event
        library = SubElement(root, 'library')
        event = SubElement(library, 'event', {
            'name': datetime.datetime.now().strftime('%Y-%m-%d')
        })
        
        # 创建project
        project = SubElement(event, 'project', {
            'name': os.path.splitext(os.path.basename(output_path))[0]
        })
        
        # 创建sequence
        sequence = SubElement(project, 'sequence', {
            'format': 'r1',
            'duration': sequence_duration,
            'tcStart': '0s',
            'tcFormat': 'DF',
            'renderFormat': 'FFRenderFormatProRes422LT',
            'audioLayout': 'stereo',
            'audioRate': '48k'
        })
        
        # 创建spine
        spine = SubElement(sequence, 'spine')
        
        # 添加音频片段
        audio_clip = SubElement(spine, 'asset-clip', {
            'ref': 'r2',
            'offset': '0s',
            'name': os.path.basename(self.audio_path),
            'duration': sequence_duration,
            'audioRole': 'dialogue'
        })
        
        # 添加固定节拍标记
        for i, beat_time in enumerate(beats_info['fixed_beats']):
            frames = int(round(beat_time * 48000))
            marker = SubElement(audio_clip, 'marker', {
                'start': f"{frames}/48000s",
                'duration': "1/48000s",
                'value': f'标记 {i+1}'
            })
        
        # 添加鼓点节拍标记
        drum_start = len(beats_info['fixed_beats']) + 1
        for i, beat_time in enumerate(beats_info['drum_beats']):
            frames = int(round(beat_time * 48000))
            num, den = frames, 48000
            g = gcd(num, den)
            num, den = num // g, den // g
            
            marker = SubElement(audio_clip, 'marker', {
                'start': f"{num}/{den}s",
                'duration': "1/48000s",
                'value': f'记 {drum_start + i}',
                'completed': '0'
            })
        
        # 保存文件
        try:
            xml_str = minidom.parseString(tostring(root, encoding='unicode')).toprettyxml(indent="  ")
            with open(output_path, 'w', encoding='utf-8') as f:
                f.write(xml_str)
            print(f"FCPXML 文已保存到：{os.path.abspath(output_path)}")
        except Exception as e:
            print(f"保存文件时出错: {str(e)}")