from abc import ABC, abstractmethod
import os
import hashlib
import time
import datetime
from xml.etree.ElementTree import Element, SubElement, Comment, ProcessingInstruction, tostring
from xml.dom import minidom
import math

class BaseDetector(ABC):
    def __init__(self, audio_path=None):
        self.audio_path = audio_path
        self.current_bpm = 120.0
        self.simplification_factor = 1
        self.beat_offset_count = 0
    
    @abstractmethod
    def detect_peaks(self):
        """检测音频中的峰值和节拍"""
        pass
    
    def get_displayed_beats(self, beats, apply_offset=False, apply_simplification=False):
        """应用简化和偏移到节拍
        
        Args:
            beats: 节拍列表
            apply_offset: 是否应用偏移（只用于固定节拍）
            apply_simplification: 是否应用简化（只用于固定节拍）
        """
        print(f"[BaseDetector] Processing beats: input length = {len(beats)}")
        print(f"[BaseDetector] Simplification factor: {self.simplification_factor}")
        print(f"[BaseDetector] Beat offset count: {self.beat_offset_count}")
        
        if len(beats) == 0:
            print("[BaseDetector] No beats to process")
            return []
            
        # 应用简化因子（只对固定节拍）
        if apply_simplification and self.simplification_factor > 1:
            print(f"[BaseDetector] Applying simplification factor {self.simplification_factor}")
            beats = beats[::self.simplification_factor]
            print(f"[BaseDetector] After simplification: {len(beats)} beats")
        
        # 只对固定节拍应用偏移
        if apply_offset and self.beat_offset_count != 0:
            print(f"[BaseDetector] Applying offset {self.beat_offset_count}")
            if self.beat_offset_count > 0:
                # 正数偏移：从指定位置开始
                if self.beat_offset_count < len(beats):
                    beats = beats[self.beat_offset_count:]
                    print(f"[BaseDetector] After positive offset: {len(beats)} beats")
                else:
                    print("[BaseDetector] Warning: Positive offset larger than number of beats")
                    return []
            else:
                # 负数偏移：从末尾开始计数
                offset = abs(self.beat_offset_count)
                if offset < len(beats):
                    beats = beats[:-offset] if offset > 0 else beats
                    print(f"[BaseDetector] After negative offset: {len(beats)} beats")
                else:
                    print("[BaseDetector] Warning: Negative offset larger than number of beats")
                    return []
        
        # 确保返回列表
        result = beats.tolist() if hasattr(beats, 'tolist') else list(beats)
        print(f"[BaseDetector] Final output: {len(result)} beats")
        return result
    
    def detect_time_signature(self, beats):
        """检测拍号
        
        基于音乐理论的拍号检测算法，通过分析以下特征：
        1. 重音模式：分析节拍的强度变化规律
        2. 节奏分组：识别自然的节奏分组
        3. 速度特征：考虑不同速度范围下常见的拍号
        
        Args:
            beats: 节拍时间点列表
            
        Returns:
            str: 检测到的拍号（例如"4/4"）
        """
        if len(beats) < 4:
            return "4/4"  # 默认拍号
            
        # 1. 基础节拍分析
        intervals = []
        for i in range(len(beats)-1):
            interval = float(beats[i+1] - beats[i])
            intervals.append(interval)
            
        avg_interval = sum(intervals) / len(intervals)
        std_dev = (sum((x - avg_interval) ** 2 for x in intervals) / len(intervals)) ** 0.5
        
        # 2. 重音模式分析
        # 通过寻找强拍位置来确定节奏模式
        accent_positions = []
        for i in range(1, len(intervals)):
            # 如果当前间隔明显大于前一个间隔，可能是强拍
            if intervals[i] > intervals[i-1] * 1.1:
                accent_positions.append(i)
            # 如果当前间隔接近平均值，而前后间隔都较小，也可能是强拍
            elif (intervals[i] > avg_interval * 0.9 and 
                  (i == 0 or intervals[i-1] < avg_interval * 0.8) and 
                  (i == len(intervals)-1 or intervals[i+1] < avg_interval * 0.8)):
                accent_positions.append(i)
        
        # 3. 分析重音之间的距离
        if accent_positions:
            accent_intervals = []
            for i in range(len(accent_positions)-1):
                accent_intervals.append(accent_positions[i+1] - accent_positions[i])
            
            # 获取最常见的重音间隔
            if accent_intervals:
                most_common_interval = max(set(accent_intervals), key=accent_intervals.count)
            else:
                most_common_interval = 4  # 默认4拍
        else:
            most_common_interval = 4  # 默认4拍
            
        # 4. 速度特征分析
        tempo_category = "medium"
        if self.current_bpm < 90:
            tempo_category = "slow"
        elif self.current_bpm > 160:
            tempo_category = "fast"
            
        # 5. 判断是否为复合拍子
        # 复合拍子通常在中速到快速时更常见
        is_compound = False
        if tempo_category in ["medium", "fast"]:
            # 检查是否存在三连音模式
            triplet_pattern = any(
                abs(intervals[i] / intervals[i+1] - 1.5) < 0.2
                for i in range(len(intervals)-1)
            )
            is_compound = triplet_pattern
            
        # 6. 根据分析结果确定拍号
        if is_compound:
            if most_common_interval <= 2:
                return "6/8"  # 两个重音，每组三个音符
            elif most_common_interval == 3:
                return "9/8"  # 三个重音，每组三个音符
            elif most_common_interval >= 4:
                return "12/8"  # 四个重音，每组三个音符
        else:
            if tempo_category == "slow":
                # 慢速音乐更可能使用较小的拍号
                if most_common_interval <= 2:
                    return "2/4"
                elif most_common_interval == 3:
                    return "3/4"
                else:
                    return "4/4"
            elif tempo_category == "fast":
                # 快速音乐通常使用2拍或4拍
                return "2/4" if most_common_interval == 2 else "4/4"
            else:
                # 中速音乐最常用4/4拍
                if most_common_interval <= 2:
                    return "2/4"
                elif most_common_interval == 3:
                    return "3/4"
                elif most_common_interval <= 6:
                    return "4/4"
                else:
                    return "4/4"  # 默认使用4/4拍

    def export_fcpxml(self, beats_info, output_path):
        """导出FCPXML文件"""
        try:
            from xml.etree.ElementTree import Element, SubElement, Comment, tostring
            from xml.dom import minidom
            import datetime
            import hashlib
            from math import gcd
            
            # 设置帧率和时间格式
            frame_duration = "1001/30000s"  # 29.97fps
            
            # 计算音频时长（根据最后一个标记点）
            max_time = max(max(beats_info['fixed_beats'] or [0]), max(beats_info['drum_beats'] or [0]))
            total_samples = int(44100 * max_time)
            
            # 计算视频帧时长 (29.97fps)
            total_frames = int(round(total_samples * 30000/(44100 * 1001)))
            sequence_duration = f"{total_frames * 1001}/30000s"
            
            # 创建FCPXML根元素
            root = Element('fcpxml', {'version': '1.11'})
            root.append(Comment('<!DOCTYPE fcpxml>'))
            
            # 添加resources
            resources = SubElement(root, 'resources')
            
            # 添加格式定义
            format_elem = SubElement(resources, 'format', {
                'id': 'r1',
                'name': 'FFVideoFormat3840x2160p2997',
                'frameDuration': frame_duration,
                'width': '3840',
                'height': '2160',
                'colorSpace': '1-1-1 (Rec. 709)'
            })
            
            # 添加音频文件资源
            audio_file = os.path.abspath(self.audio_path)
            uid = hashlib.md5(audio_file.encode()).hexdigest().upper()
            
            asset = SubElement(resources, 'asset', {
                'id': 'r2',
                'name': os.path.basename(self.audio_path),
                'uid': uid,
                'start': '0s',
                'duration': f"{total_samples}/44100s",
                'hasAudio': '1',
                'audioSources': '1',
                'audioChannels': '2',
                'audioRate': '44100'
            })
            
            # 添加 media-rep
            media_rep = SubElement(asset, 'media-rep', {
                'kind': 'original-media',
                'sig': uid,
                'src': f"file://{audio_file}"
            })
            
            # 添加 metadata
            metadata = SubElement(asset, 'metadata')
            md = SubElement(metadata, 'md', {
                'key': 'com.apple.proapps.mio.ingestDate',
                'value': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S +0800')
            })
            
            # 添加library和event
            library = SubElement(root, 'library')
            event = SubElement(library, 'event', {
                'name': datetime.datetime.now().strftime('%Y-%m-%d')
            })
            
            # 创建project
            project = SubElement(event, 'project', {
                'name': os.path.splitext(os.path.basename(output_path))[0]
            })
            
            # 创建sequence
            sequence = SubElement(project, 'sequence', {
                'format': 'r1',
                'duration': sequence_duration,
                'tcStart': '0s',
                'tcFormat': 'DF',
                'renderFormat': 'FFRenderFormatProRes422LT',
                'audioLayout': 'stereo',
                'audioRate': '48k'
            })
            
            # 创建spine
            spine = SubElement(sequence, 'spine')
            
            # 添加音频片段
            audio_clip = SubElement(spine, 'asset-clip', {
                'ref': 'r2',
                'offset': '0s',
                'name': os.path.basename(self.audio_path),
                'duration': sequence_duration,
                'audioRole': 'dialogue'
            })
            
            # 添加固定节拍标记
            for i, beat_time in enumerate(beats_info['fixed_beats']):
                frames = int(round(beat_time * 48000))
                marker = SubElement(audio_clip, 'marker', {
                    'start': f"{frames}/48000s",
                    'duration': "1/48000s",
                    'value': f'标记 {i+1}'
                })
            
            # 添加鼓点节拍标记
            drum_start = len(beats_info['fixed_beats']) + 1
            for i, beat_time in enumerate(beats_info['drum_beats']):
                frames = int(round(beat_time * 48000))
                num, den = frames, 48000
                g = gcd(num, den)
                num, den = num // g, den // g
                
                marker = SubElement(audio_clip, 'marker', {
                    'start': f"{num}/{den}s",
                    'duration': "1/48000s",
                    'value': f'记 {drum_start + i}',
                    'completed': '0'
                })
            
            # 保存文件
            try:
                xml_str = minidom.parseString(tostring(root, encoding='unicode')).toprettyxml(indent="  ")
                with open(output_path, 'w', encoding='utf-8') as f:
                    f.write(xml_str)
                print(f"FCPXML 文件已保存到：{os.path.abspath(output_path)}")
            except Exception as e:
                print(f"保存文件时出错: {str(e)}")
        except Exception as e:
            print(f"保存文件时出错: {str(e)}")
