import numpy as np
import essentia.standard as es
from .base_detector import BaseDetector

class EssentiaDetector(BaseDetector):
    def __init__(self, audio_path=None):
        super().__init__(audio_path)
        self.audio = None
        if audio_path:
            self.audio = es.MonoLoader(filename=audio_path)()
    
    def detect_peaks(self):
        """使用Essentia检测音频中的峰值和节拍"""
        # 1. 使用节奏提取器获取基本节拍信息
        rhythm_extractor = es.RhythmExtractor2013()
        bpm, beats, confidence, estimates, bpm_intervals = rhythm_extractor(self.audio)
        self.current_bpm = bpm

        # 2. 使用多种onset检测方法
        frame_size = 2048
        hop_size = 512

        # 创建处理链
        w = es.Windowing(type='hann')
        fft = es.FFT()
        c2p = es.CartesianToPolar()
        
        # 使用不同的onset检测器
        onset_functions = {
            'hfc': es.OnsetDetection(method='hfc'),
            'complex': es.OnsetDetection(method='complex'),
            'flux': es.OnsetDetection(method='flux')
        }
        
        onset_values = {method: [] for method in onset_functions.keys()}
        
        # 处理每一帧
        for frame in es.FrameGenerator(self.audio, frameSize=frame_size, hopSize=hop_size):
            mag, phase = c2p(fft(w(frame)))
            
            # 对每种方法计算onset值
            for method, detector in onset_functions.items():
                onset_values[method].append(float(detector(mag, phase)))
        
        # 转换为numpy数组以便处理
        for method in onset_values:
            onset_values[method] = np.array(onset_values[method])
        
        # 对每种方法找到显著的onset
        all_onsets = set()
        for method, values in onset_values.items():
            # 计算自适应阈值
            threshold = np.mean(values) + 0.5 * np.std(values)
            
            # 找到超过阈值的位置
            peaks = np.where(values > threshold)[0]
            
            # 转换为时间
            times = peaks * hop_size / 44100.0
            all_onsets.update(times)
        
        # 排序并移除太近的onset
        onset_times = sorted(list(all_onsets))
        filtered_onsets = []
        last_onset = -float('inf')
        min_interval = 0.05  # 50ms
        
        for onset in onset_times:
            if onset - last_onset >= min_interval:
                filtered_onsets.append(onset)
                last_onset = onset
        
        # 检测拍号
        time_signature = self.detect_time_signature(beats)
        
        # 应用简化和偏移到固定节拍，不对鼓点节拍应用简化和偏移
        fixed_beats = self.get_displayed_beats(beats, apply_offset=True, apply_simplification=True)
        drum_beats = self.get_displayed_beats(filtered_onsets, apply_offset=False, apply_simplification=False)
        
        return {
            'tempo': float(self.current_bpm),
            'peaks': filtered_onsets,
            'fixed_beats': fixed_beats,
            'drum_beats': drum_beats,
            'beat_type': time_signature,
            'confidence': confidence,
            'simplification_factor': self.simplification_factor,
            'beat_offset_count': self.beat_offset_count
        }

    def get_displayed_beats(self, beats, apply_offset=True, apply_simplification=True):
        """获取实际显示的节拍点"""
        if not isinstance(beats, np.ndarray):
            beats = np.array(beats)
            
        if beats.size == 0:
            return []
            
        # 计算节拍间隔
        beat_interval = 60.0 / self.current_bpm
        
        # 处理向前偏移超出范围的情况
        if self.beat_offset_count < 0:
            # 计算需要添加的前置节拍点数
            extra_beats_count = abs(self.beat_offset_count)
            # 生成前置节拍点
            first_beat = beats[0]
            extra_beats = np.array([first_beat - (i + 1) * beat_interval 
                                  for i in range(extra_beats_count)])
            # 确保不会出现负数时间点
            extra_beats = extra_beats[extra_beats >= 0]
            # 合并节拍点
            beats = np.concatenate([extra_beats[::-1], beats])  # 反转前置节拍点以保持正确顺序
            
        # 应用偏移
        if apply_offset and self.beat_offset_count != 0:
            if self.beat_offset_count > 0:
                # 正向偏移：从指定位置开始
                beats = beats[self.beat_offset_count:]
            else:
                # 负向偏移：已经在上面处理过了
                pass
            
        # 应用简化
        if apply_simplification and self.simplification_factor > 1:
            beats = beats[::self.simplification_factor]
            
        return beats.tolist()
