import numpy as np
import librosa
from .base_detector import BaseDetector

class LibrosaDetector(BaseDetector):
    def __init__(self, audio_path=None):
        super().__init__(audio_path)
        self.onset_threshold = 0.07  # onset检测的阈值
        self.min_tempo = 60
        self.max_tempo = 200
        
    def set_preset(self, preset):
        """设置预设参数"""
        if preset == 'default':
            self.onset_threshold = 0.07
            self.min_tempo = 60
            self.max_tempo = 200
        elif preset == 'electronic':
            self.onset_threshold = 0.05
            self.min_tempo = 120
            self.max_tempo = 140
        elif preset == 'rock':
            self.onset_threshold = 0.08
            self.min_tempo = 90
            self.max_tempo = 180
        elif preset == 'pop':
            self.onset_threshold = 0.07
            self.min_tempo = 80
            self.max_tempo = 130
        elif preset == 'jazz':
            self.onset_threshold = 0.09
            self.min_tempo = 70
            self.max_tempo = 140
        elif preset == 'classical':
            self.onset_threshold = 0.1
            self.min_tempo = 50
            self.max_tempo = 120
        elif preset == 'hiphop':
            self.onset_threshold = 0.07
            self.min_tempo = 85
            self.max_tempo = 110

    def detect_peaks(self):
        """使用Librosa检测音频中的峰值和节拍"""
        # 加载音频
        y, sr = librosa.load(self.audio_path)
        
        # 1. 使用librosa进行节拍检测，考虑tempo范围
        tempo, beats = librosa.beat.beat_track(
            y=y, 
            sr=sr,
            start_bpm=self.min_tempo,
            tightness=100,
            trim=False,
            units='time'
        )
        
        # 如果检测到的速度超出范围，尝试重新检测
        if tempo < self.min_tempo or tempo > self.max_tempo:
            tempo = (self.min_tempo + self.max_tempo) / 2
            beats = librosa.beat.beat_track(
                y=y, 
                sr=sr,
                bpm=tempo,
                tightness=100,
                trim=False,
                units='time'
            )[1]

        self.current_bpm = float(tempo)
        
        # 2. 使用onset检测方法
        onset_env = librosa.onset.onset_strength(y=y, sr=sr)
        onset_frames = librosa.onset.onset_detect(
            onset_envelope=onset_env,
            sr=sr,
            units='time',
            hop_length=512,
            backtrack=True,
            delta=self.onset_threshold,  # 使用预设的阈值
            wait=30
        )
        
        # 应用简化因子到固定节拍
        if len(beats) > 0:
            # 转换为numpy数组以便进行切片操作
            beats = np.array(beats)
            
            # 应用简化和偏移到固定节拍，不对鼓点节拍应用简化和偏移
            fixed_beats = self.get_displayed_beats(beats, apply_offset=True, apply_simplification=True)
            drum_beats = self.get_displayed_beats(onset_frames, apply_offset=False, apply_simplification=False)
        else:
            fixed_beats = []
            drum_beats = []
        
        # 检测拍号
        time_signature = self.detect_time_signature(beats)
        
        # 确保所有返回的数值都是Python原生类型
        return {
            'tempo': float(self.current_bpm),
            'peaks': [float(t) for t in onset_frames],
            'fixed_beats': [float(t) for t in fixed_beats],
            'drum_beats': [float(t) for t in drum_beats],
            'beat_type': time_signature,
            'confidence': 1.0,
            'simplification_factor': self.simplification_factor,
            'beat_offset_count': self.beat_offset_count,
            'time_signature': time_signature
        }

    def get_displayed_beats(self, beats, apply_offset, apply_simplification):
        if apply_offset:
            # 计算节拍间隔
            beat_interval = 60.0 / self.current_bpm
            
            # 处理向前偏移超出范围的情况
            if self.beat_offset_count < 0:
                # 计算需要添加的前置节拍点数
                extra_beats_count = abs(self.beat_offset_count)
                # 生成前置节拍点
                first_beat = beats[0]
                extra_beats = np.array([first_beat - (i + 1) * beat_interval 
                                      for i in range(extra_beats_count)])
                # 确保不会出现负数时间点
                extra_beats = extra_beats[extra_beats >= 0]
                # 合并节拍点
                beats = np.concatenate([extra_beats[::-1], beats])  # 反转前置节拍点以保持正确顺序
            
            # 应用偏移
            if self.beat_offset_count != 0:
                if self.beat_offset_count > 0:
                    # 正向偏移：从指定位置开始
                    beats = beats[self.beat_offset_count:]
                else:
                    # 负向偏移：已经在上面处理过了
                    pass
                
            # 应用简化
            if apply_simplification and self.simplification_factor > 1:
                beats = beats[::self.simplification_factor]
        return beats.tolist()
