import numpy as np

# Fix for numpy deprecation of np.float and np.int
import numpy
if not hasattr(numpy, 'float'):
    numpy.float = float
if not hasattr(numpy, 'int'):
    numpy.int = int

from madmom.features.beats import RNNBeatProcessor, DBNBeatTrackingProcessor
from madmom.features.tempo import TempoEstimationProcessor
from .base_detector import BaseDetector

class MadmomDetector(BaseDetector):
    def __init__(self, audio_path=None):
        super().__init__(audio_path)
        self.min_tempo = 60
        self.max_tempo = 200
        
    def set_preset(self, preset):
        """设置预设参数"""
        if preset == 'default':
            self.min_tempo = 60
            self.max_tempo = 200
        elif preset == 'electronic':
            self.min_tempo = 120
            self.max_tempo = 140
        elif preset == 'rock':
            self.min_tempo = 90
            self.max_tempo = 180
        elif preset == 'pop':
            self.min_tempo = 80
            self.max_tempo = 130
        elif preset == 'jazz':
            self.min_tempo = 70
            self.max_tempo = 140
        elif preset == 'classical':
            self.min_tempo = 50
            self.max_tempo = 120
        elif preset == 'hiphop':
            self.min_tempo = 85
            self.max_tempo = 110

    def detect_peaks(self):
        """使用Madmom检测音频中的峰值和节拍"""
        print("[Madmom] Starting beat detection")
        
        # 1. 使用RNN进行节拍检测
        proc = DBNBeatTrackingProcessor(fps=100)
        act = RNNBeatProcessor()(self.audio_path)
        beats = proc(act)
        print(f"[Madmom] Initial beats detected: {len(beats)}")
        
        # 2. 使用TempoEstimationProcessor进行速度检测
        tempo_proc = TempoEstimationProcessor(fps=100)
        tempo_estimates = tempo_proc(act)
        
        # 获取最可能的速度
        self.current_bpm = float(tempo_estimates[0][0])  # 取第一个速度估计
        confidence = float(tempo_estimates[0][1])  # 取第一个速度估计的置信度
        print(f"[Madmom] Detected BPM: {self.current_bpm}, Confidence: {confidence}")
        
        # 3. 使用检测到的节拍作为基准点
        if len(beats) > 0:
            print(f"[Madmom] Using detected beats as reference")
            # 使用检测到的第一个节拍作为起点
            start_time = beats[0]
            # 计算节拍间隔（秒）
            beat_interval = 60.0 / self.current_bpm
            
            # 计算音频总长度（秒）
            duration = float(len(act)) / 100.0
            print(f"[Madmom] Audio duration: {duration}s, Beat interval: {beat_interval}s")
            
            # 从第一个检测到的节拍开始，生成固定间隔的节拍
            num_beats = int((duration - start_time) / beat_interval) + 1
            fixed_beats = np.array([start_time + i * beat_interval for i in range(num_beats)])
            print(f"[Madmom] Generated {len(fixed_beats)} fixed beats starting from {start_time}s")
        else:
            print(f"[Madmom] No beats detected, generating from start")
            # 如果没有检测到节拍，从0开始生成
            beat_interval = 60.0 / self.current_bpm
            duration = float(len(act)) / 100.0
            print(f"[Madmom] Audio duration: {duration}s, Beat interval: {beat_interval}s")
            num_beats = int(duration / beat_interval) + 1
            fixed_beats = np.array([i * beat_interval for i in range(num_beats)])
            print(f"[Madmom] Generated {len(fixed_beats)} fixed beats from 0s")
        
        # 4. 生成鼓点（使用节拍激活函数的局部最大值）
        peaks = []
        window_size = 3
        threshold = np.max(act) * 0.3  # 降低阈值以检测更多鼓点
        
        for i in range(window_size, len(act) - window_size):
            if act[i] > threshold and all(act[i] >= act[j] for j in range(i-window_size, i+window_size+1)):
                peaks.append(i / 100.0)  # 转换为秒
        
        drum_beats = np.array(peaks)
        print(f"[Madmom] Detected {len(drum_beats)} drum beats")
        
        # 5. 应用简化和偏移
        print(f"[Madmom] Before simplification: {len(fixed_beats)} fixed beats, factor: {self.simplification_factor}, offset: {self.beat_offset_count}")
        fixed_beats = self.get_displayed_beats(fixed_beats, apply_offset=True, apply_simplification=True)
        drum_beats = self.get_displayed_beats(drum_beats, apply_offset=False, apply_simplification=False)
        print(f"[Madmom] After simplification: {len(fixed_beats)} fixed beats")
        
        # 6. 检测拍号
        beat_type = self.detect_time_signature(fixed_beats)
        print(f"[Madmom] Detected time signature: {beat_type}")
        
        result = {
            'tempo': self.current_bpm,
            'peaks': peaks,
            'fixed_beats': fixed_beats,  # 已经是列表了
            'drum_beats': drum_beats,    # 已经是列表了
            'beat_type': beat_type,
            'confidence': confidence
        }
        print(f"[Madmom] Final result: {len(result['fixed_beats'])} fixed beats, {len(result['drum_beats'])} drum beats")
        return result
